import type { Attachment } from '../../../../decorators/attachment/Attachment';
import type { LinkedAttachment } from '../../../../utils/LinkedAttachment';
import type { ParseRevocationRegistryDefinitionTemplate } from '../../../ledger/services';
import type { AutoAcceptCredential } from '../../CredentialAutoAcceptType';
import type { CredentialPreviewAttribute, CredentialPreviewAttributeOptions } from '../../models/CredentialPreviewAttribute';
import type { V2CredentialPreview } from '../../protocol/v2/V2CredentialPreview';
import type { CredentialExchangeRecord } from '../../repository/CredentialExchangeRecord';
import type { CredPropose } from './CredPropose';
import { CredentialFormatType } from '../../CredentialsModuleOptions';
export declare type CredentialFormats = FormatServiceOfferCredentialFormats | FormatServiceProposeCredentialFormats | FormatServiceRequestCredentialFormats;
export interface IndyCredentialPreview {
    credentialDefinitionId?: string;
    attributes?: CredentialPreviewAttribute[];
}
export interface IndyProposeCredentialFormat {
    attributes?: CredentialPreviewAttributeOptions[];
    linkedAttachments?: LinkedAttachment[];
    payload?: CredPropose;
}
export interface IndyOfferCredentialFormat {
    credentialDefinitionId: string;
    attributes: CredentialPreviewAttribute[];
    linkedAttachments?: LinkedAttachment[];
}
export interface IndyRequestCredentialFormat {
    credentialDefinitionId?: string;
    attributes?: CredentialPreviewAttribute[];
}
export interface IndyIssueCredentialFormat {
    credentialDefinitionId?: string;
    attributes?: CredentialPreviewAttribute[];
}
export declare class CredentialFormatSpec {
    constructor(options: {
        attachId: string;
        format: string;
    });
    attachId: string;
    format: string;
}
export declare type FormatKeys = {
    [id: string]: CredentialFormatType;
};
export interface FormatServiceCredentialAttachmentFormats {
    format: CredentialFormatSpec;
    attachment: Attachment;
}
export interface FormatServiceProposeAttachmentFormats extends FormatServiceCredentialAttachmentFormats {
    preview?: V2CredentialPreview;
}
export interface FormatServiceOfferAttachmentFormats extends FormatServiceCredentialAttachmentFormats {
    preview?: V2CredentialPreview;
}
export declare const FORMAT_KEYS: FormatKeys;
export interface FormatServiceOfferCredentialFormats {
    indy?: IndyOfferCredentialFormat;
    jsonld?: undefined;
}
export interface FormatServiceProposeCredentialFormats {
    indy?: IndyProposeCredentialFormat;
    jsonld?: undefined;
}
export interface FormatServiceAcceptProposeCredentialFormats {
    indy?: {
        credentialDefinitionId?: string;
        attributes: CredentialPreviewAttributeOptions[];
        linkedAttachments?: LinkedAttachment[];
    };
    jsonld?: undefined;
}
export interface FormatServiceRequestCredentialFormats {
    indy?: IndyRequestCredentialFormat;
    jsonld?: undefined;
}
export interface FormatServiceIssueCredentialFormats {
    indy?: IndyIssueCredentialFormat;
    jsonld?: undefined;
}
export interface HandlerAutoAcceptOptions {
    credentialRecord: CredentialExchangeRecord;
    autoAcceptType: AutoAcceptCredential;
    messageAttributes?: CredentialPreviewAttribute[];
    proposalAttachment?: Attachment;
    offerAttachment?: Attachment;
    requestAttachment?: Attachment;
    credentialAttachment?: Attachment;
    credentialDefinitionId?: string;
}
export interface RevocationRegistry {
    indy?: ParseRevocationRegistryDefinitionTemplate;
    jsonld?: undefined;
}
