"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.V2IssueCredentialHandler = void 0;
const helpers_1 = require("../../../../../agent/helpers");
const AriesFrameworkError_1 = require("../../../../../error/AriesFrameworkError");
const V2IssueCredentialMessage_1 = require("../messages/V2IssueCredentialMessage");
const V2RequestCredentialMessage_1 = require("../messages/V2RequestCredentialMessage");
class V2IssueCredentialHandler {
    constructor(credentialService, agentConfig, didCommMessageRepository) {
        this.supportedMessages = [V2IssueCredentialMessage_1.V2IssueCredentialMessage];
        this.credentialService = credentialService;
        this.agentConfig = agentConfig;
        this.didCommMessageRepository = didCommMessageRepository;
    }
    async handle(messageContext) {
        const credentialRecord = await this.credentialService.processCredential(messageContext);
        const credentialMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: V2IssueCredentialMessage_1.V2IssueCredentialMessage,
        });
        const requestMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: V2RequestCredentialMessage_1.V2RequestCredentialMessage,
        });
        if (!credentialMessage) {
            throw new AriesFrameworkError_1.AriesFrameworkError(`Missing credential message from credential record ${credentialRecord.id}`);
        }
        const shouldAutoRespond = this.credentialService.shouldAutoRespondToCredential(credentialRecord, credentialMessage);
        if (shouldAutoRespond) {
            return await this.createAck(credentialRecord, messageContext, requestMessage !== null && requestMessage !== void 0 ? requestMessage : undefined, credentialMessage);
        }
    }
    async createAck(record, messageContext, requestMessage, credentialMessage) {
        this.agentConfig.logger.info(`Automatically sending acknowledgement with autoAccept on ${this.agentConfig.autoAcceptCredentials}`);
        const { message } = await this.credentialService.createAck(record);
        if (messageContext.connection) {
            return helpers_1.createOutboundMessage(messageContext.connection, message);
        }
        else if ((requestMessage === null || requestMessage === void 0 ? void 0 : requestMessage.service) && (credentialMessage === null || credentialMessage === void 0 ? void 0 : credentialMessage.service)) {
            const recipientService = credentialMessage.service;
            const ourService = requestMessage.service;
            return helpers_1.createOutboundServiceMessage({
                payload: message,
                service: recipientService.resolvedDidCommService,
                senderKey: ourService.resolvedDidCommService.recipientKeys[0],
            });
        }
        this.agentConfig.logger.error(`Could not automatically create credential ack`);
    }
}
exports.V2IssueCredentialHandler = V2IssueCredentialHandler;
//# sourceMappingURL=V2IssueCredentialHandler.js.map