"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.V2ProposeCredentialHandler = void 0;
const helpers_1 = require("../../../../../agent/helpers");
const V2ProposeCredentialMessage_1 = require("../messages/V2ProposeCredentialMessage");
class V2ProposeCredentialHandler {
    constructor(credentialService, agentConfig, didCommMessageRepository) {
        this.supportedMessages = [V2ProposeCredentialMessage_1.V2ProposeCredentialMessage];
        this.credentialService = credentialService;
        this.agentConfig = agentConfig;
        this.didCommMessageRepository = didCommMessageRepository;
    }
    async handle(messageContext) {
        const credentialRecord = await this.credentialService.processProposal(messageContext);
        const handlerOptions = {
            credentialRecord,
            autoAcceptType: this.agentConfig.autoAcceptCredentials,
        };
        const shouldAutoRespond = await this.credentialService.shouldAutoRespondToProposal(handlerOptions);
        if (shouldAutoRespond) {
            return await this.createOffer(credentialRecord, messageContext);
        }
    }
    async createOffer(credentialRecord, messageContext) {
        this.agentConfig.logger.info(`Automatically sending offer with autoAccept on ${this.agentConfig.autoAcceptCredentials}`);
        if (!messageContext.connection) {
            this.agentConfig.logger.error('No connection on the messageContext, aborting auto accept');
            return;
        }
        const message = await this.credentialService.createOfferAsResponse(credentialRecord);
        return helpers_1.createOutboundMessage(messageContext.connection, message);
    }
}
exports.V2ProposeCredentialHandler = V2ProposeCredentialHandler;
//# sourceMappingURL=V2ProposeCredentialHandler.js.map